'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _color = require('color');

var _color2 = _interopRequireDefault(_color);

var _cssColorNames = require('css-color-names');

var _cssColorNames2 = _interopRequireDefault(_cssColorNames);

var _postcss = require('postcss');

var _postcss2 = _interopRequireDefault(_postcss);

var _postcssValueParser = require('postcss-value-parser');

var _postcssValueParser2 = _interopRequireDefault(_postcssValueParser);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var plugin = 'postcss-color-yiq';

function err(node) {
    throw node.parent.error('No background color was found.', { plugin: plugin });
}

function getYIQContrast(colour, _ref) {
    var light = _ref.light;
    var dark = _ref.dark;

    var _color$rgbArray = (0, _color2.default)(colour).rgbArray();

    var r = _color$rgbArray[0];
    var g = _color$rgbArray[1];
    var b = _color$rgbArray[2];

    var yiq = (r * 299 + g * 587 + b * 114) / 1000;
    return yiq >= 128 ? dark : light;
}

exports.default = _postcss2.default.plugin(plugin, function (opts) {
    opts = _extends({
        dark: '#000',
        light: '#fff'
    }, opts);
    return function (css) {
        css.walkDecls('color', function (decl) {
            if (decl.value !== 'yiq') {
                return;
            }

            var background = decl.parent.nodes.filter(function (_ref2) {
                var prop = _ref2.prop;

                return prop === 'background' || prop === 'background-color';
            }).pop();

            if (typeof background === 'undefined') {
                err(decl);
            }

            var hasColour = false;

            decl.value = (0, _postcssValueParser2.default)(background.value).walk(function (node) {
                if (node.type === 'function' && /^(rgb|hsl)a?$/.test(node.value)) {
                    node.value = getYIQContrast((0, _postcssValueParser.stringify)(node), opts);
                    node.type = 'word';
                    hasColour = true;
                    return;
                }
                if (node.type === 'word') {
                    if (!node.value.indexOf('#')) {
                        node.value = getYIQContrast(node.value, opts);
                        hasColour = true;
                        return;
                    } else if (node.value in _cssColorNames2.default) {
                        node.value = getYIQContrast(_cssColorNames2.default[node.value], opts);
                        hasColour = true;
                        return;
                    }
                }
                node.value = '';
                node.nodes = null;
            }).toString();

            if (!hasColour) {
                decl.value = 'yiq';
                err(decl);
            }
        });
    };
});
module.exports = exports['default'];